// Pie chart visualization of google/wiki results

class PieChart
{
  
  // Empty Constructor
  PieChart(){}
  
  // Draw pie chart
  // @param percent percentage of each element (should sum 1)
  // @param colors color of each element
  // @param diameter circle diameter
  // @param start start angle (degrees)
  //
  // @note The char is drawn counter-clockwise, starting from the start angle
  // Function similar to: http://processing.org/learning/basics/piechart.html
  void draw(float[] percent, color[] colors, float diameter, float start)
  {
      float lastAng = start, nextAng = start;
      float radious = diameter/2.0;
      start = radians(start);
      for (int p=0; p<percent.length; p++)
      {
         // convert percent to angles
         nextAng += radians(percent[p]*360.0);
         fill(colors[p]);
         noStroke();
         arc(radious, radious, diameter, diameter, lastAng, nextAng);
         lastAng = nextAng;
      }
  }
  
  // Draw a pie chart for google and wiki results
  // @param goog WordPair result from Google
  // @param wiki WordPair result from Wikipedia
  // @param colors elements colors (should be 4)
  // @param diameter circle diameter
  // @param start start angle (degrees)
  //
  // @note The char is drawn counter-clockwise, starting from the start angle
  void draw(WordPair goog, WordPair wiki, color[] colors, float diameter, float start)
  {
       float[] percent = {0.0, 0.0, 0.0, 0.0};
       long sumWiki = wiki.misPop + wiki.repPop;
       long sumGoog = goog.misPop + goog.repPop;
       percent[0] = (float)wiki.misPop/sumWiki;
       percent[1] = (float)wiki.repPop/sumWiki;
       percent[2] = (float)(goog.repPop - percent[1])/sumGoog;
       percent[3] = (float)(goog.misPop - percent[0])/sumGoog;
       draw(percent, colors, diameter, start);
  }
  
   
  // Draw a pie chart for google and wiki results with default colors and starting angle
  // @param goog WordPair result from Google
  // @param wiki WordPair result from Wikipedia
  // @param diameter circle diameter
  //
  // @note The char is drawn counter-clockwise, starting from the start angle 
  void draw(WordPair goog, WordPair wiki, float diameter)
  {
     color[] colors = {ORANGEDARK, GREY, GREYLIGHT, ORANGE};
     float start = 0.0; 
     draw(goog, wiki, colors, diameter, start);
  }
  
  // Draw a pie chart for google and wiki results
  // @param wordpair WordPair result
  // @param colors elements colors (should be 2)
  // @param start start angle (degrees)
  // @param diameter circle diameter
  //
  // @note The char is drawn counter-clockwise, starting from the start angle
  void draw(WordPair wordpair, color[] colors, float diameter, float start)
  {
       float[] percent = {0.0, 0.0};
       long sumwp = wordpair.misPop + wordpair.repPop;
       percent[0] = (float)wordpair.misPop/sumwp;
       percent[1] = (float)wordpair.repPop/sumwp;
       draw(percent, colors, diameter, start);
  }
  
  // Draw a pie chart for google and wiki results
  // @param wordpair WordPair result
  // @param diameter circle diameter
  //
  // @note The char is drawn counter-clockwise, starting from the start angle
  void draw(WordPair wordpair, float diameter)
  {
     color[] colors = {ORANGEDARK, BLUE};
     float start = 0.0; 
     draw(wordpair, colors, diameter, start);
  }
  
  
}
