
// WordPair container
// Each element in the hashmap is an ArrayList, each key is a letter of the alphabet
class ResultsSet
{
  
  HashMap hashmap;
  HashMap links;
  
  // Constructor
  ResultsSet()
  {
     hashmap = new HashMap(); 
     links = new HashMap();
  }
  
  // Fill the hashmap with ArrayLists of WordPairs.
  // Each ArrayList corresponds to the initial letter of all WordPairs contained.
  // The key of each element in the HashMap is the initial letter of the ArrayList elements.
  //
  // @param hashmap where to store the data
  // @param filename name of file to load from the Data folder
  void loadResultPairs(String filename)
  {
    String filepath = dataPath(filename);
    try {
      BufferedReader file = new BufferedReader(new FileReader(filepath));
      String linestr = file.readLine();
      while( linestr != null )
      {
        String[] tok = splitTokens(linestr, "|");
        WordPair pair = new WordPair(tok[0],Long.parseLong(tok[1]),
                                     tok[2],Long.parseLong(tok[3]));                            
        String initial = tok[0].substring(0,1);
        ArrayList arraylist;
        if (!hashmap.containsKey(initial))
        {
          arraylist = new ArrayList();
        }
        else
        {
          arraylist = (ArrayList)hashmap.get(initial);
        }
        arraylist.add(pair);
        hashmap.put(initial,arraylist);
        linestr = file.readLine();
      }
      
    }
    catch (Exception e)
    {
      println ("Error" + e) ;
    }
    
  }
  
  void loadLinks(PApplet parent, ArrayList misspellings)
  {

    for (int m=0; m<misspellings.size(); m++)
    {
      WordPair wp = (WordPair)misspellings.get(m);
      
    String xmlFileName = new String(wp.mis);
    String initial = xmlFileName.substring(0,1);
    if (initial.equals(initial.toLowerCase()))
     {
      xmlFileName += "L.xml";
     } 
     else
    {
     xmlFileName += "C.xml";   
    }
    String xmlFile = dataPath(xmlFileName);
    
    ArrayList linkslist = new ArrayList();
    
    XMLElement xml = new XMLElement(parent, xmlFile); 
    int numPages = xml.getChildCount();
    for (int i = 0; i < numPages && i < 3; i++) {
      XMLElement page = xml.getChild(i);
      
      String title = "";
      String description = "";
      String url ="";
      
      for (int p = 0; p < page.getChildCount(); p++) {
        XMLElement child = page.getChild(p);
         if (child.getName().equals("title"))
           title = child.getContent();
         else if (child.getName().equals("description"))
            description = child.getContent();
         else if (child.getName().equals("link"))
           url = child.getContent();
      }
      Link link = new Link(title, description, url);
      linkslist.add(link);  
    }
    
    if (linkslist == null)
    {
       println("LINKSLIST is NULL"); 
    }
    links.put(wp.mis, linkslist);
    
    }
  }
  
  // Print result pairs to console
  // @param hashmap data to print
  void printResultPairs(boolean sortAlphabetically)
  {
    ArrayList keys = new ArrayList(hashmap.keySet());
    if (sortAlphabetically)
    {
       Collections.sort(keys); 
    }
    println("num keys: " + keys.size());
    Iterator iterator = keys.iterator();
    while (iterator.hasNext())
    {
        String thekey = (String)iterator.next();
        ArrayList thepairs = (ArrayList)hashmap.get(thekey);
        for (int p=0; p<thepairs.size(); p++)
       {
          WordPair pair = (WordPair)thepairs.get(p);
          println("{"+thekey+"} " + pair.mis + " (" + String.valueOf(pair.misPop) + ") " + 
                  pair.rep + " (" + String.valueOf(pair.repPop) + ")"); 
       }
    }
  }

  String getRandomKey()
  {
      ArrayList keys = new ArrayList(hashmap.keySet());
      println(random(0,keys.size()));
      int val = floor(random(0,keys.size()));
      return (String)keys.get(val);
  }
  
  ArrayList getWordPairsForKey(String thekey)
  {
       return (ArrayList)hashmap.get(thekey);
  }
  
  WordPair getWordPair(String misspelling)
  {
     String thekey = misspelling.substring(0,1);
     ArrayList arraylist = (ArrayList)hashmap.get(thekey);
     if (arraylist == null)
     {
       println("[WARNING] ResultsSet::getWordPair - Null key");
     }
 
     for (int p=0; p<arraylist.size(); p++)
     {
        WordPair wordpair = (WordPair)arraylist.get(p);
        if (wordpair.mis.equals(misspelling))
        {
           return wordpair; 
        }
     }
     return null;
  }
  
  float getWordPairRatio(String misspelling)
  {
     String thekey = misspelling.substring(0,1);
     ArrayList arraylist = (ArrayList)hashmap.get(thekey);
     if (arraylist == null)
     {
       println("[WARNING] ResultsSet::getWordPair - Null key");
     }
 
     for (int p=0; p<arraylist.size(); p++)
     {
        WordPair wordpair = (WordPair)arraylist.get(p);
        if (wordpair.mis.equals(misspelling))
        {
           return wordpair.ratio; 
        }
     }
     return -1.0;
  }

  int getNumberOfMisspellings()
  {
     int number = 0;
     ArrayList keys = new ArrayList(hashmap.keySet());
     Iterator iterator = keys.iterator();
     while (iterator.hasNext())
     {
        String thekey = (String)iterator.next();
        ArrayList thepairs = (ArrayList)hashmap.get(thekey);
        number += thepairs.size();
    }
    return number;
  }
  
  long getMaxPopMisspelling()
  {
    long maxPop = 0;
    int number = 0;
    ArrayList keys = new ArrayList(hashmap.keySet());
    Iterator iterator = keys.iterator();
    while (iterator.hasNext())
    {
        String thekey = (String)iterator.next();
        ArrayList thepairs = (ArrayList)hashmap.get(thekey);
        for (int p=0; p<thepairs.size(); p++)
        {
          WordPair pair = (WordPair)thepairs.get(p);
          if (pair.misPop > maxPop)
          {
             maxPop = pair.misPop; 
          }
        }
    }
    return maxPop;
  }
  
  long getMaxPopReplacement()
  {
    long maxPop = 0;
    int number = 0;
    ArrayList keys = new ArrayList(hashmap.keySet());
    Iterator iterator = keys.iterator();
    while (iterator.hasNext())
    {
        String thekey = (String)iterator.next();
        ArrayList thepairs = (ArrayList)hashmap.get(thekey);
        for (int p=0; p<thepairs.size(); p++)
        {
          WordPair pair = (WordPair)thepairs.get(p);
          if (pair.repPop > maxPop)
          {
             maxPop = pair.misPop; 
          }
        }
    }
    return maxPop;
  }
  
  WordPair getWordPairByKeyID(int keyid)
  {
     int number = 0, tmp = 0;
     ArrayList keys = new ArrayList(hashmap.keySet());
     Iterator iterator = keys.iterator();
     while (iterator.hasNext())
     {
        String thekey = (String)iterator.next();
        ArrayList thepairs = (ArrayList)hashmap.get(thekey);
        tmp = number + thepairs.size();
        if (tmp > keyid)
        {
          tmp = keyid - number;
          return (WordPair)thepairs.get(tmp);
        }
        number = tmp;
    }
    return null;
  }
  
  ArrayList getSortedMisspellings(WordPairSorter sorter)
  {
    ArrayList misspellings = new ArrayList();
    ArrayList keys = new ArrayList(hashmap.keySet());
    Iterator iterator = keys.iterator();
    while (iterator.hasNext())
    {
        String thekey = (String)iterator.next();
        ArrayList thepairs = (ArrayList)hashmap.get(thekey);
        for (int p=0; p<thepairs.size(); p++)
       {
          WordPair pair = (WordPair)thepairs.get(p);
          misspellings.add(pair);
       }
    }
    Collections.sort(misspellings, sorter);
    return misspellings;
  }
  
  long getSumPopularity()
  {
    long sumRatios = 0;
    ArrayList misspellings = new ArrayList();
    ArrayList keys = new ArrayList(hashmap.keySet());
    Iterator iterator = keys.iterator();
    while (iterator.hasNext())
    {
        String thekey = (String)iterator.next();
        ArrayList thepairs = (ArrayList)hashmap.get(thekey);
        for (int p=0; p<thepairs.size(); p++)
       {
          WordPair pair = (WordPair)thepairs.get(p);
          sumRatios += pair.misPop;
       }
    }
    return sumRatios;
  }
  
  ArrayList getLinks(String thekey)
  {
     return (ArrayList)links.get(thekey); 
  }

}
