// Word pair (misspelling/result and replacement/result)
class WordPair
{
  
  String mis;          //misspelling
  String rep;          //replacement
  long misPop;          //misspelling popularity (# results)
  long repPop;          //replacement popularity (# results)
  float ratio;
  
  WordPair(String m, long mpop, String r, long rpop)
  {
     mis = m;
     misPop = mpop;
     rep = r;
     repPop = rpop; 
     ratio = (float)misPop/repPop;
  }
  
  WordPair(WordPair w)
  {
     mis = w.mis;
     misPop = w.misPop;
     rep = w.rep;
     repPop = w.repPop; 
     ratio = w.ratio;
  }
  
}

// Word pair sorter
abstract class WordPairSorter implements Comparator<WordPair>
{
   abstract int compare(WordPair w1, WordPair w2);
   abstract float getMetric(WordPair w);
}

// Compare using misPop/repPop ratio
class RatioComparator extends WordPairSorter 
{
  int compare(WordPair w1, WordPair w2)
  {
    if ((w2.ratio - w1.ratio) > 0) return 1;
    else if ((w2.ratio - w1.ratio) < 0) return -1;
    else return 0;
  }
  
  float getMetric(WordPair w)
  {
     return w.ratio;
  }  
}

// Compare using misPop
class MisPopComparator extends WordPairSorter 
{
  int compare(WordPair w1, WordPair w2)
  {
    if ((w2.misPop - w1.misPop) > 0) return 1;
    else if ((w2.misPop - w1.misPop) < 0) return -1;
    else return 0;
  }
  
  float getMetric(WordPair w)
  {
     return (float)w.misPop/1000.0;
  }  
}

class RandomComparator extends WordPairSorter 
{
  int compare(WordPair w1, WordPair w2)
  {
    if (random(0,1) > 0.5) return 1;
    else return -1; 
  }
  
  float getMetric(WordPair w)
  {
     return random(0,1);
  }   
}
